#include <libmemcached/memcached.h>
#include <libmemcached/allocators.h>
#include <libmemcached/get.h>
#include <libmemcached/fetch.h>
#include <libmemcached/stats.h>
#include <libmemcached/dump.h>
#include <libmemcached/server.h>
#include <ch.h>

static ChInterp_t interp;

////////////////////////
//memcached_dump_chdl
////////////////////////
static void* memcached_dump_chdl_funptr;
static memcached_return_t memcached_dump_chdl_funarg(const memcached_st *ptr, const char *key, size_t key_length, void *context)
{
  memcached_return_t retval;
  Ch_CallFuncByAddr(interp, memcached_dump_chdl_funptr, &retval, ptr, key, key_length, context);
  return retval;
}


EXPORTCH memcached_return_t memcached_dump_chdl(void *varg) {
    //ChInterp_t interp;
    ChVaList_t ap;
    memcached_st *ptr;
    memcached_dump_fn *function_ch, *function_c = NULL;
    void *context;
    uint32_t number_of_callbacks;
    memcached_return_t retval;

    Ch_VaStart(interp, ap, varg);
    ptr = Ch_VaArg(interp, ap, memcached_st *);
    function_ch = Ch_VaArg(interp, ap, memcached_dump_fn*);
    
    memcached_dump_chdl_funptr = (void*)function_ch;
    if (function_ch!=NULL)
      function_c = (memcached_dump_fn*)memcached_dump_chdl_funarg;
    
    context = Ch_VaArg(interp, ap, void *);
    number_of_callbacks = Ch_VaArg(interp, ap, uint32_t);
    retval = memcached_dump(ptr, function_c, context, number_of_callbacks);
    Ch_VaEnd(interp, ap);
    return retval;
}




///////////////////////////
//memcached_set_memory_allocators_chdl
///////////////////////////

static void* memcached_malloc_function_chdl_funptr;
static void* memcached_free_function_chdl_funptr;
static void* memcached_realloc_function_chdl_funptr;
static void* memcached_calloc_function_chdl_funptr;

static void* memcached_malloc_function_chdl_funarg(const memcached_st *ptr, const size_t size, void *context)
{
  void* retval;
  Ch_CallFuncByAddr(interp, memcached_malloc_function_chdl_funptr, &retval, ptr, size, context);
  return retval;
}

static void memcached_free_function_chdl_funarg(memcached_st *ptr, void *mem, void *context)
{
  Ch_CallFuncByAddr(interp, memcached_free_function_chdl_funptr, ptr, mem, context);
}

static void* memcached_realloc_function_chdl_funarg(const memcached_st *ptr, void *mem, const size_t size, void *context)
{
  void* retval;
  Ch_CallFuncByAddr(interp, memcached_realloc_function_chdl_funptr, &retval, ptr, mem, size, context);
  return retval;
}

static void* memcached_calloc_function_chdl_funarg(const memcached_st *ptr, size_t nelem, const size_t elsize, void *context)
{
  void* retval;
  Ch_CallFuncByAddr(interp, memcached_calloc_function_chdl_funptr, &retval, ptr, nelem, elsize, context);
  return retval;
}


EXPORTCH memcached_return_t memcached_set_memory_allocators_chdl(void *varg) {
    //ChInterp_t interp;
    ChVaList_t ap;
    memcached_st *ptr;
    memcached_malloc_fn mem_malloc_CH, mem_malloc_C=NULL;
    memcached_free_fn mem_free_CH, mem_free_C=NULL;
    memcached_realloc_fn mem_realloc_CH, mem_realloc_C=NULL;
    memcached_calloc_fn mem_calloc_CH, mem_calloc_C=NULL;
    void *context;
    memcached_return_t retval;

    Ch_VaStart(interp, ap, varg);
    ptr = Ch_VaArg(interp, ap, memcached_st *);
    
    mem_malloc_CH = Ch_VaArg(interp, ap, memcached_malloc_fn);
    memcached_malloc_function_chdl_funptr = (void*)mem_malloc_CH;
    if (mem_malloc_CH!=NULL)
      mem_malloc_C = (memcached_malloc_fn)memcached_malloc_function_chdl_funarg;
    
    mem_free_CH = Ch_VaArg(interp, ap, memcached_free_fn);
    memcached_free_function_chdl_funptr = (void*)mem_free_CH;
    if (mem_free_CH!=NULL)
      mem_free_C = (memcached_free_fn)memcached_free_function_chdl_funarg;
    
    mem_realloc_CH = Ch_VaArg(interp, ap, memcached_realloc_fn);
    memcached_realloc_function_chdl_funptr = (void*)mem_realloc_CH;
    if (mem_realloc_CH!=NULL)
      mem_realloc_C = (memcached_realloc_fn)memcached_realloc_function_chdl_funarg;
    
    mem_calloc_CH = Ch_VaArg(interp, ap, memcached_calloc_fn);
    memcached_calloc_function_chdl_funptr = (void*)mem_calloc_CH;
    if (mem_calloc_CH!=NULL)
      mem_calloc_C = (memcached_calloc_fn)memcached_calloc_function_chdl_funarg;

    context = Ch_VaArg(interp, ap, void *);
    
    retval = memcached_set_memory_allocators(ptr, mem_malloc_C, mem_free_C, mem_realloc_C, mem_calloc_C, context);
    Ch_VaEnd(interp, ap);
    return retval;
}


////////////////////////
//memcached_get_memory_allocators_chdl
////////////////////////

EXPORTCH void memcached_get_memory_allocators_chdl(void *varg) {
    //ChInterp_t interp;
    ChVaList_t ap;
    memcached_st *ptr;
    memcached_malloc_fn *mem_malloc_CH, *mem_malloc_C=NULL;
    memcached_free_fn *mem_free_CH, *mem_free_C=NULL;
    memcached_realloc_fn *mem_realloc_CH, *mem_realloc_C=NULL;
    memcached_calloc_fn *mem_calloc_CH, *mem_calloc_C=NULL;

    Ch_VaStart(interp, ap, varg);
    ptr = Ch_VaArg(interp, ap, memcached_st *);
    mem_malloc_CH = Ch_VaArg(interp, ap, memcached_malloc_fn*);
    memcached_malloc_function_chdl_funptr = (void*)mem_malloc_CH;
    if (mem_malloc_CH!=NULL)
      mem_malloc_C = (memcached_malloc_fn*)memcached_malloc_function_chdl_funarg;
    
    mem_free_CH = Ch_VaArg(interp, ap, memcached_free_fn*);
    memcached_free_function_chdl_funptr = (void*)mem_free_CH;
    if (mem_free_CH!=NULL)
      mem_free_C = (memcached_free_fn*)memcached_free_function_chdl_funarg;
    
    mem_realloc_CH = Ch_VaArg(interp, ap, memcached_realloc_fn*);
    memcached_realloc_function_chdl_funptr = (void*)mem_realloc_CH;
    if (mem_realloc_CH!=NULL)
      mem_realloc_C = (memcached_realloc_fn*)memcached_realloc_function_chdl_funarg;
    
    mem_calloc_CH = Ch_VaArg(interp, ap, memcached_calloc_fn*);
    memcached_calloc_function_chdl_funptr = (void*)mem_calloc_CH;
    if (mem_calloc_CH!=NULL)
      mem_calloc_C = (memcached_calloc_fn*)memcached_calloc_function_chdl_funarg;
    
    memcached_get_memory_allocators(ptr, mem_malloc_C, mem_free_C, mem_realloc_C, mem_calloc_C);
    Ch_VaEnd(interp, ap);
}


//////////////////
//memcached_fetch_execute_chdl
//////////////////

static void* memcached_execute_function_chdl_funptr;
static memcached_return_t memcached_execute_function_chdl_funarg(const memcached_st *ptr, memcached_result_st *result, void *context)
{
  memcached_return_t retval;
  Ch_CallFuncByAddr(interp, memcached_execute_function_chdl_funptr, &retval, ptr, result, context);
  return retval;
}

EXPORTCH memcached_return_t memcached_fetch_execute_chdl(void *varg) {
    //ChInterp_t interp;
    ChVaList_t ap;
    memcached_st *ptr;
    memcached_execute_fn *callback_CH, *callback_C=NULL;
    void *context;
    unsigned int number_of_callbacks;
    memcached_return_t retval;

    Ch_VaStart(interp, ap, varg);
    ptr = Ch_VaArg(interp, ap, memcached_st *);
    
    callback_CH = Ch_VaArg(interp, ap, memcached_execute_fn*);
    memcached_execute_function_chdl_funptr = (void*)callback_CH;
    if (callback_CH!=NULL)
      callback_C = (memcached_execute_fn*)memcached_execute_function_chdl_funarg;
    
    context = Ch_VaArg(interp, ap, void *);
    number_of_callbacks = Ch_VaArg(interp, ap, unsigned int);
    retval = memcached_fetch_execute(ptr, callback_C, context, number_of_callbacks);
    Ch_VaEnd(interp, ap);
    return retval;
}



///////////////////
//memcached_server_cursor_chdl
///////////////////

static void* memcached_server_function_chdl_funptr;
static memcached_return_t memcached_server_function_chdl_funarg(const memcached_st *ptr, memcached_server_instance_st server, void *context)
{
  memcached_return_t retval;
  Ch_CallFuncByAddr(interp, memcached_server_function_chdl_funptr, &retval, ptr, server, context);
  return retval;
}

EXPORTCH memcached_return_t memcached_server_cursor_chdl(void *varg) {
    //ChInterp_t interp;
    ChVaList_t ap;
    memcached_st *ptr;
    memcached_server_fn *callback_CH, *callback_C = NULL;
    void *context;
    unsigned int number_of_callbacks;
    memcached_return_t retval;

    Ch_VaStart(interp, ap, varg);
    ptr = Ch_VaArg(interp, ap, memcached_st *);
    
    callback_CH = Ch_VaArg(interp, ap, memcached_server_fn*);
    memcached_server_function_chdl_funptr = (void*)callback_CH;
    if (callback_CH!=NULL)
      callback_C = (memcached_server_fn*)memcached_server_function_chdl_funarg;

    
    context = Ch_VaArg(interp, ap, void *);
    number_of_callbacks = Ch_VaArg(interp, ap, unsigned int);
    retval = memcached_server_cursor(ptr, callback_C, context, number_of_callbacks);
    Ch_VaEnd(interp, ap);
    return retval;
}



///////////////////
//memcached_mget_execute
///////////////////

static void* memcached_execute_fn_chdl_funptr;
static memcached_return_t memcached_execute_fn_chdl_funarg(const memcached_st *ptr, memcached_result_st *result, void *context)
{
  memcached_return_t retval;
  Ch_CallFuncByAddr(interp, memcached_execute_fn_chdl_funptr, &retval, ptr, result, context);
  return retval;
}


EXPORTCH memcached_return_t memcached_mget_execute_chdl(void *varg) {
    ChInterp_t interp;
    ChVaList_t ap;
    memcached_st *ptr;
    char *const *keys;
    size_t *key_length;
    size_t number_of_keys;
    memcached_execute_fn *callback_CH, *callback_C;
    void *context;
    uint32_t number_of_callbacks;
    memcached_return_t retval;

    Ch_VaStart(interp, ap, varg);
    ptr = Ch_VaArg(interp, ap, memcached_st *);
    keys = Ch_VaArg(interp, ap, char *const *);
    key_length = Ch_VaArg(interp, ap, size_t *);
    number_of_keys = Ch_VaArg(interp, ap, size_t);
    
    callback_CH = Ch_VaArg(interp, ap, memcached_execute_fn*);
    memcached_execute_fn_chdl_funptr = (void*)callback_CH;
    if (callback_CH!=NULL)
      callback_C = (memcached_execute_fn*)memcached_execute_fn_chdl_funarg;
    
    context = Ch_VaArg(interp, ap, void *);
    number_of_callbacks = Ch_VaArg(interp, ap, uint32_t);
    retval = memcached_mget_execute(ptr, keys, key_length, number_of_keys, callback_C, context, number_of_callbacks);
    Ch_VaEnd(interp, ap);
    return retval;
}


///////////////////
//memcached_mget_execute_by_key_chdl
///////////////////
EXPORTCH memcached_return_t memcached_mget_execute_by_key_chdl(void *varg) {
    ChInterp_t interp;
    ChVaList_t ap;
    memcached_st *ptr;
    char *group_key;
    size_t group_key_length;
    char *const *keys;
    size_t *key_length;
    size_t number_of_keys;
    memcached_execute_fn *callback_CH, *callback_C;
    void *context;
    uint32_t number_of_callbacks;
    memcached_return_t retval;

    Ch_VaStart(interp, ap, varg);
    ptr = Ch_VaArg(interp, ap, memcached_st *);
    group_key = Ch_VaArg(interp, ap, char *);
    group_key_length = Ch_VaArg(interp, ap, size_t);
    keys = Ch_VaArg(interp, ap, char *const *);
    key_length = Ch_VaArg(interp, ap, size_t *);
    number_of_keys = Ch_VaArg(interp, ap, size_t);
    
    callback_CH = Ch_VaArg(interp, ap, memcached_execute_fn*);
    memcached_execute_fn_chdl_funptr = (void*)callback_CH;
    if (callback_CH!=NULL)
      callback_C = (memcached_execute_fn*)memcached_execute_fn_chdl_funarg;
    
    context = Ch_VaArg(interp, ap, void *);
    number_of_callbacks = Ch_VaArg(interp, ap, uint32_t);
    retval = memcached_mget_execute_by_key(ptr, group_key, group_key_length, keys, key_length, number_of_keys, callback_C, context, number_of_callbacks);
    Ch_VaEnd(interp, ap);
    return retval;
}



///////////////////
//memcached_stat_execute
///////////////////
static void* memcached_stat_fn_chdl_funptr;
static memcached_return_t memcached_stat_fn_chdl_funarg(memcached_server_instance_st server, const char *key, size_t key_length, const char *value, size_t value_length, void *context)
{
  memcached_return_t retval;
  Ch_CallFuncByAddr(interp, memcached_stat_fn_chdl_funptr, &retval, server, key, key_length, value, value_length, context);
  return retval;
}


EXPORTCH memcached_return_t memcached_stat_execute_chdl(void *varg) {
    ChInterp_t interp;
    ChVaList_t ap;
    memcached_st *memc;
    char *args;
    memcached_stat_fn func_CH, func_C;
    void *context;
    memcached_return_t retval;

    Ch_VaStart(interp, ap, varg);
    memc = Ch_VaArg(interp, ap, memcached_st *);
    args = Ch_VaArg(interp, ap, char *);
    
    func_CH = Ch_VaArg(interp, ap, memcached_stat_fn);
    memcached_stat_fn_chdl_funptr = (void*)func_CH;
    if (func_CH!=NULL)
      func_C = (memcached_stat_fn)memcached_stat_fn_chdl_funarg;
    
    context = Ch_VaArg(interp, ap, void *);
    retval = memcached_stat_execute(memc, args, func_C, context);
    Ch_VaEnd(interp, ap);
    return retval;
}



