#!/bin/ch

// this script will generate the ch package

#include <unistd.h>   // for access()
#include <dlfcn.h>    // for dlopen()
#include <chshell.h>  // for chinfo()

int addChdlHeader(char *headerfile, char *macro, char *chdlheader);

string_t pkgname="chmemcached";          // define package name
string_t libname="memcached";              // assume the lib is libsql.so
string_t libhashname="hashkit";              // assume the lib is libsql.so  

string_t lib_header_dir;             // library header files directory 
string_t lib_hash_header_dir;             // library header files directory 
string_t createchf_dir="chfcreate/"; // library directory containing chf files
string_t createchf_hash_dir="chfcreate/hashkit/"; // library directory containing chf files
string_t c_dir ="c/";                // directory  to generate .dl file
string_t c_hash_dir ="c/libhashkit/";
string_t include_dir="include/libmemcached/";     // header files used for Ch
string_t include_hash_dir="include/libhashkit/";     // header files used for Ch  
string_t demo_dir="demos/";          // directory  to generate .dl file
string_t cwd = _cwd;                 // the current working directory

chinfo_t info;

//make sure pgkcreate.ch is run from the current working directory
if(access("pkgcreate.ch", R_OK)) {
    echo Run ./pkgcreate.ch in the current directory.
    exit(-1);
}

// make sure you are running ch version workable with this script.
chinfo(&info);
if ((info.vermajor*100+ info.verminor*10 +  info.vermicro) < 501) {
   echo "To run this script, you need to install Ch version 5.0.1.12201 or higher"
   echo "You can download the latest version from http://www.softintegration.com/download"
   exit(-1);
}

/*  
    we assume the library header files are either 
    in /usr/local/include or /usr/include 
*/
if (access("/usr/local/include/libmemcached/memcached.h", R_OK ) == 0)
{
	echo "we assume memcache header files are located in /usr/local/include";
	lib_header_dir="/usr/local/include/libmemcached";
	lib_hash_header_dir="/usr/local/include/libhashkit";
}
else if  (access("/usr/include/libmemcached/memcached.h", R_OK ) == 0)
{
	echo "we assume memcache header files are located in /usr/include";
	lib_header_dir = "/usr/include/libmemcached";
	lib_hash_header_dir="/usr/include/libhashkit";
}
else
{
	echo "Error: unable to find memcached header files.";
        echo "Please download memcached source code and install it first.";
	exit(-1);
}

/* clean up existing chunixodbc directory  */
if (access(pkgname, R_OK) ==0)
{
        echo "Clean up package directory ${pkgname} now";
        rm -rf $pkgname;
        rm -rf $include_dir
}

// create the directory for distribution
mkdir $pkgname $pkgname/dl $pkgname/lib $pkgname/lib/hashkit $pkgname/include $pkgname/include/libmemcached $pkgname/include/libhashkit $pkgname/demos
mkdir -p $include_dir
mkdir -p $include_hash_dir
mkdir -p $c_hash_dir
mkdir -p $createchf_hash_dir

echo copy library header files for ch
//cp $lib_header_dir/memcached.h $include_dir 
cp $lib_header_dir/*.h $include_dir 
cp $lib_hash_header_dir/*.h $include_hash_dir 

/**/


sed -i 's/#pragma once/#ifndef __MEMCACHED_H__\n#define __MEMCACHED_H__/' "include/libmemcached/memcached.h"
sed -i -e '$a#endif' "include/libmemcached/memcached.h"
sed -i 's/#pragma once/#ifndef __allocators_H__\n#define __allocators_H__/' "include/libmemcached/allocators.h"
sed -i -e '$a#endif' "include/libmemcached/allocators.h"
sed -i 's/#pragma once/#ifndef __analyze_H__\n#define __analyze_H__/' "include/libmemcached/analyze.h"
sed -i -e '$a#endif' "include/libmemcached/analyze.h"
sed -i 's/#pragma once/#ifndef __array_H__\n#define __array_H__/' "include/libmemcached/array.h"
sed -i -e '$a#endif' "include/libmemcached/array.h"
sed -i 's/#pragma once/#ifndef __auto_H__\n#define __auto_H__/' "include/libmemcached/auto.h"
sed -i -e '$a#endif' "include/libmemcached/auto.h"
sed -i 's/#pragma once/#ifndef __basic_string_H__\n#define __basic_string_H__/' "include/libmemcached/basic_string.h"
sed -i -e '$a#endif' "include/libmemcached/basic_string.h"
sed -i 's/#pragma once/#ifndef __behavior_H__\n#define __behavior_H__/' "include/libmemcached/behavior.h"
sed -i -e '$a#endif' "include/libmemcached/behavior.h"
sed -i 's/#pragma once/#ifndef __callback_H__\n#define __callback_H__/' "include/libmemcached/callback.h"
sed -i -e '$a#endif' "include/libmemcached/callback.h"
sed -i 's/#pragma once/#ifndef __delete_H__\n#define __delete_H__/' "include/libmemcached/delete.h"
sed -i -e '$a#endif' "include/libmemcached/delete.h"
sed -i 's/#pragma once/#ifndef __dump_H__\n#define __dump_H__/' "include/libmemcached/dump.h"
sed -i -e '$a#endif' "include/libmemcached/dump.h"
sed -i 's/#pragma once/#ifndef __error_H__\n#define __error_H__/' "include/libmemcached/error.h"
sed -i -e '$a#endif' "include/libmemcached/error.h"
sed -i 's/#pragma once/#ifndef __fetch_H__\n#define __fetch_H__/' "include/libmemcached/fetch.h"
sed -i -e '$a#endif' "include/libmemcached/fetch.h"
sed -i 's/#pragma once/#ifndef __flush_buffers_H__\n#define __flush_buffers_H__/' "include/libmemcached/flush_buffers.h"
sed -i -e '$a#endif' "include/libmemcached/flush_buffers.h"
sed -i 's/#pragma once/#ifndef __flush_H__\n#define __flush_H__/' "include/libmemcached/flush.h"
sed -i -e '$a#endif' "include/libmemcached/flush.h"
sed -i 's/#pragma once/#ifndef __hash_H__\n#define __hash_H__/' "include/libmemcached/hash.h"
sed -i -e '$a#endif' "include/libmemcached/hash.h"
sed -i 's/#pragma once/#ifndef __namespace_H__\n#define __namespace_H__/' "include/libmemcached/namespace.h"
sed -i -e '$a#endif' "include/libmemcached/namespace.h"
sed -i 's/#pragma once/#ifndef __parse_H__\n#define __parse_H__/' "include/libmemcached/parse.h"
sed -i -e '$a#endif' "include/libmemcached/parse.h"
sed -i 's/#pragma once/#ifndef __quit_H__\n#define __quit_H__/' "include/libmemcached/quit.h"
sed -i -e '$a#endif' "include/libmemcached/quit.h"
sed -i 's/#pragma once/#ifndef __result_H__\n#define __result_H__/' "include/libmemcached/result.h"
sed -i -e '$a#endif' "include/libmemcached/result.h"
sed -i 's/#pragma once/#ifndef __server_H__\n#define __server_H__/' "include/libmemcached/server.h"
sed -i -e '$a#endif' "include/libmemcached/server.h"
sed -i 's/#pragma once/#ifndef __server_list_H__\n#define __server_list_H__/' "include/libmemcached/server_list.h"
sed -i -e '$a#endif' "include/libmemcached/server_list.h"
sed -i 's/#pragma once/#ifndef __stats_H__\n#define __stats_H__/' "include/libmemcached/stats.h"
sed -i -e '$a#endif' "include/libmemcached/stats.h"
sed -i 's/#pragma once/#ifndef __storage_H__\n#define __storage_H__/' "include/libmemcached/storage.h"
sed -i -e '$a#endif' "include/libmemcached/storage.h"
sed -i 's/#pragma once/#ifndef __strerror_H__\n#define __strerror_H__/' "include/libmemcached/strerror.h"
sed -i -e '$a#endif' "include/libmemcached/strerror.h"
sed -i 's/#pragma once/#ifndef __string_H__\n#define __string_H__/' "include/libmemcached/string.h"
sed -i -e '$a#endif' "include/libmemcached/string.h"
sed -i 's/#pragma once/#ifndef __version_H__\n#define __version_H__/' "include/libmemcached/version.h"
sed -i -e '$a#endif' "include/libmemcached/version.h"


sed -i 's/#pragma once/#ifndef __hashkit_H__\n#define __hashkit_H__/' "include/libhashkit/hashkit.h"
sed -i -e '$a#endif' "include/libhashkit/hashkit.h"
sed -i 's/#pragma once/#ifndef __str_algorithm_H__\n#define __str_algorithm_H__/' "include/libhashkit/str_algorithm.h"
sed -i -e '$a#endif' "include/libhashkit/str_algorithm.h"


// needs special handling for sql.h file used for Ch
char *macro ="#define __MEMCACHED_H__";

string_t chdlheader =stradd(
  "\n\n",
  "#if defined(_CH_) \n"
  "#pragma package <", pkgname, "> \n",
  "#include <chdl.h> \n",
  "LOAD_CHDL(", libname, ")\n",
  "#endif \n");

// copy the library header file with the change
// it will add chdlheader under "#define _SQL_H"; in sql.h
addChdlHeader(stradd(include_dir, "memcached.h"), macro, chdlheader);

echo generate header files containing exported functions only
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "memcached.h"), stradd(createchf_dir, "memcached.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "allocators.h"), stradd(createchf_dir, "allocators.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "analyze.h"), stradd(createchf_dir, "analyze.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "array.h"), stradd(createchf_dir, "array.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "auto.h"), stradd(createchf_dir, "auto.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "behavior.h"), stradd(createchf_dir, "behavior.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "callback.h"), stradd(createchf_dir, "callback.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "delete.h"), stradd(createchf_dir, "delete.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "dump.h"), stradd(createchf_dir, "dump.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "error.h"), stradd(createchf_dir, "error.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "fetch.h"), stradd(createchf_dir, "fetch.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "flush_buffers.h"), stradd(createchf_dir, "flush_buffers.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "flush.h"), stradd(createchf_dir, "flush.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "get.h"), stradd(createchf_dir, "get.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "hash.h"), stradd(createchf_dir, "hash.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "namespace.h"), stradd(createchf_dir, "namespace.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "parse.h"), stradd(createchf_dir, "parse.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "quit.h"), stradd(createchf_dir, "quit.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "result.h"), stradd(createchf_dir, "result.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "server.h"), stradd(createchf_dir, "server.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "server_list.h"), stradd(createchf_dir, "server_list.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "stats.h"), stradd(createchf_dir, "stats.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "storage.h"), stradd(createchf_dir, "storage.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "strerror.h"), stradd(createchf_dir, "strerror.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "string.h"), stradd(createchf_dir, "string.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "version.h"), stradd(createchf_dir, "version.h"), NULL);

//these functions are handmade as arguments - pointer to functions
removeFuncProto("chfcreate/allocators.h", "memcached_set_memory_allocators", 0);
removeFuncProto("chfcreate/allocators.h", "memcached_get_memory_allocators", 0);
removeFuncProto("chfcreate/fetch.h", "memcached_fetch_execute", 0);
removeFuncProto("chfcreate/get.h", "memcached_mget_execute", 0);
removeFuncProto("chfcreate/get.h", "memcached_mget_execute_by_key", 0);
removeFuncProto("chfcreate/server.h", "memcached_server_cursor", 0);
removeFuncProto("chfcreate/stats.h", "memcached_stat_execute", 0);
removeFuncProto("chfcreate/dump.h", "memcached_dump", 0);

echo generated header files
//exit(-1);

echo generating *.chf and *_chdl.c files
echo process memcache.h
c2chf $createchf_dir/memcached.h -i c libmemcached/memcached.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/allocators.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/allocators.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/analyze.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/analyze.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/array.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/array.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/auto.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/auto.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/behavior.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/behavior.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/callback.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/callback.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/delete.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/delete.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/dump.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/dump.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/error.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/error.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/fetch.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/fetch.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/flush_buffers.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/flush_buffers.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/flush.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/flush.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/get.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/get.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/hash.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/hash.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/namespace.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/namespace.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/parse.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/parse.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/quit.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/quit.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/result.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/result.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/server.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/server.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/server_list.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/server_list.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/stats.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/stats.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/storage.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/storage.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/strerror.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/strerror.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/string.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/string.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/version.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/version.h -o c ${c_dir} -o chf ${pkgname}/lib

///////hashkit
/*
string_t chdlheader2 =stradd(
  "\n\n",
  "#if defined(_CH_) \n"
  "#pragma package <", pkgname, "> \n",
  "#include <chdl.h> \n",
  "LOAD_CHDL(", libhashname, ")\n",
  "#endif \n");

// copy the library header file with the change
// it will add chdlheader under "#define _SQL_H"; in sql.h
addChdlHeader(stradd(include_hash_dir, "hashkit.h"), macro, chdlheader2);
*/
echo generate header files containing exported functions only
processhfile("HASHKIT_API", 0,  ";",   stradd(include_hash_dir, "hashkit.h"), stradd(createchf_hash_dir, "hashkit.h"), NULL);
processhfile("HASHKIT_API", 0,  ";",   stradd(include_hash_dir, "algorithm.h"), stradd(createchf_hash_dir, "algorithm.h"), NULL);
processhfile("HASHKIT_API", 0,  ";",   stradd(include_hash_dir, "digest.h"), stradd(createchf_hash_dir, "digest.h"), NULL);
processhfile("HASHKIT_API", 0,  ";",   stradd(include_hash_dir, "function.h"), stradd(createchf_hash_dir, "function.h"), NULL);
processhfile("HASHKIT_API", 0,  ";",   stradd(include_hash_dir, "str_algorithm.h"), stradd(createchf_hash_dir, "str_algorithm.h"), NULL);
processhfile("HASHKIT_API", 0,  ";",   stradd(include_hash_dir, "strerror.h"), stradd(createchf_hash_dir, "strerror.h"), NULL);

//these functions are handmade as arguments - pointer to functions
removeFuncProto("chfcreate/hashkit/function.h", "hashkit_set_custom_function", 0);
removeFuncProto("chfcreate/hashkit/function.h", "hashkit_set_custom_distribution_function", 0);
removeFuncProto("chfcreate/hashkit/algorithm.h", "libhashkit_hsieh", 0);

echo generated header files
//exit(-1);

echo generating *.chf and *_chdl.c files
echo process hashkit.h
c2chf $createchf_hash_dir/hashkit.h -i c libhashkit/hashkit.h -o c ${c_hash_dir} -o chf ${pkgname}/lib/hashkit
c2chf $createchf_hash_dir/algorithm.h -h _Ch${libhashname}_handle -i c libhashkit/hashkit.h -i c libhashkit/algorithm.h -o c ${c_hash_dir} -o chf ${pkgname}/lib/hashkit
c2chf $createchf_hash_dir/digest.h -h _Ch${libhashname}_handle -i c libhashkit/hashkit.h -i c libhashkit/digest.h -o c ${c_hash_dir} -o chf ${pkgname}/lib/hashkit
c2chf $createchf_hash_dir/function.h -h _Ch${libhashname}_handle -i c libhashkit/hashkit.h -i c libhashkit/function.h -o c ${c_hash_dir} -o chf ${pkgname}/lib/hashkit
c2chf $createchf_hash_dir/str_algorithm.h -h _Ch${libhashname}_handle -i c libhashkit/hashkit.h -i c libhashkit/str_algorithm.h -o c ${c_hash_dir} -o chf ${pkgname}/lib/hashkit
c2chf $createchf_hash_dir/strerror.h -h _Ch${libhashname}_handle -i c libhashkit/hashkit.h -i c libhashkit/strerror.h -o c ${c_hash_dir} -o chf ${pkgname}/lib/hashkit

echo patching special .chf files ...
cp -f chfhandmade/*.chf $pkgname/lib
cp -f chfhandmade/*.c $c_dir
cp -f chfhandmade/hashkit/*.chf $pkgname/lib/hashkit
cp -f chfhandmade/hashkit/*.c $c_dir/libhashkit
/**/
cd ${c_dir}
echo clean up ${c_dir} object files
make clean
make

echo test if .dl file is created successfully
string_t libmemcachedl = stradd(`pwd`, "/libmemcached.dl");

if ( strcmp(`uname`,"Linux") ==0 )
{
    chcon -t texrel_shlib_t $libmemcachedl  >/dev/null 2>&1
}

if ( dlopen(libmemcachedl, 1) == NULL)
{
    echo "error in libmemcached.dl";
    printf("%s \n", dlerror());
    exit(-1);
}


echo test if .dl file is created successfully
string_t libhashkitdl = stradd(`pwd`, "/libhashkit.dl");

if ( strcmp(`uname`,"Linux") ==0 )
{
    chcon -t texrel_shlib_t $libhashkitdl  >/dev/null 2>&1
}

if ( dlopen(libhashkitdl, 1) == NULL)
{
    echo "error in libhashkit.dl";
    printf("%s \n", dlerror());
    exit(-1);
}



// go back to previous directory
cd $cwd

echo "libmemcache.dl is successfully created";

echo copy the created files
cp $c_dir/lib${libname}.dl $pkgname/dl/
cp $include_dir/*.h $pkgname/include/libmemcached
cp -p $demo_dir/* $pkgname/demos/
cp $c_dir/lib${libhashname}.dl $pkgname/dl/
cp $include_hash_dir/*.h $pkgname/include/libhashkit

echo "${pkgname} is successfully created";

/*******************************************************************
 * * This function will add the required preprocessing directives
 * * needed by Ch to a header file.
 * * headerfile:   header file to be changed
 * * macro:        token to be recoganized for insertion of chdlheader
 * * chdlheader:   preprocessing directives to be inserted
 * *******************************************************************/
int addChdlHeader(char *headerfile, char *macro, char *chdlheader)
{
   char *stop_ptr;
   string_t tempfile, token;
   string_t result;

   tempfile = ``cat $headerfile``;
   token = strstr( tempfile, macro);
   stop_ptr = strstr(token, "\n");
   strncpy(result, tempfile, strlen(tempfile)-strlen(stop_ptr));
   result=stradd(result, chdlheader);
   result=stradd(result, stop_ptr);
   
   FILE *stream;
   if(!(stream = fopen("_tmpfile", "w")))
   {
      perror("_tmpfile");
      return -1;
   }
   fprintf(stream, "%s", result);
   fclose(stream);
   mv -f _tmpfile $(headerfile)

   return 0;

}
